@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Contact the author at:
  *              Jens Berger
@@ -232,41 +232,50 @@ DEBUGBITS(Debug_BIT)
  MSG(pmNoOpenDir,  etWarn, TRUE, 0, \
      "Could not open the directory `%s'.") \
  MSG(pmNoDebugFlag, etWarn, TRUE, 0, \
-     "This version of doesn't support the -d flag. Please recompile.")
+     "This version of doesn't support the -d flag. Please recompile.")\
+ MSG(pmNoRegexMem,  etErr,  TRUE, 0,\
+     "Unable to allocate regular expressions - no memory?\n")\
+ MSG(pmRegexCompileFailed,  etWarn,  TRUE, 0,\
+     "Compilation of regular expression %s failed at offset %d with error %s.\n")\
+ MSG(pmRegexMatchingError,  etErr,  TRUE, 0,\
+     "PCRE matching error %d.\n")
+
 #undef MSG
 #define MSG(num, type, inuse, ctxt, text) num,
-     enum PrgErrNum
-     {
-         PRGMSGS pmMaxFault
-     };
+enum PrgErrNum
+{
+    PRGMSGS pmMaxFault
+};
 #undef MSG
 
-     extern struct ErrMsg PrgMsgs[pmMaxFault + 1];
+extern struct ErrMsg PrgMsgs[pmMaxFault + 1];
 
-     struct ErrInfo
-     {
-         char *Data;
-         const char *LineBuf;
-         char *File;
-         unsigned long Line, Column, ErrLen;
-         enum
-         {
-             efNone = 0x00,
-             efNoItal = 0x01,
-             efItal = 0x02
-         } Flags;
-     };
+struct ErrInfo
+{
+    char *Data;
+    const char *LineBuf;
+    const char *File;
+    unsigned long Line, Column, ErrLen;
+    enum
+    {
+        efNone   = 0x00,
+        efNoItal = 0x01,
+        efItal   = 0x02,
+        efNoMath = 0x04,
+        efMath   = 0x08
+    } Flags;
+};
 
-     extern char *ReadBuffer;
-     extern char *CmdBuffer;
-     extern char *TmpBuffer;
+extern char *ReadBuffer;
+extern char *CmdBuffer;
+extern char *TmpBuffer;
 
-     extern const char BrOrder[NUMBRACKETS + 1];
+extern const char BrOrder[NUMBRACKETS + 1];
 
-     extern unsigned long Brackets[NUMBRACKETS];
+extern unsigned long Brackets[NUMBRACKETS];
 
 
-     extern FILE *OutputFile, *InputFile;
+extern FILE *OutputFile, *InputFile;
 
 #define OPTION_DEFAULTS \
   DEF(int, GlobalRC,    TRUE) \
@@ -277,12 +286,13 @@ DEBUGBITS(Debug_BIT)
   DEF(int, UsingStdIn,  FALSE) \
   DEF(int, InputFiles,  TRUE) \
   DEF(int, HeadErrOut,  TRUE) \
-  DEF(char *, OutputName, "") \
-  DEF(char *, PseudoInName, NULL) \
+  DEF(const char *, OutputName, "") \
+  DEF(const char *, PseudoInName, NULL) \
   DEF(char *, OutputFormat, VerbNormal) \
   DEF(char *, PipeOutputFormat, NULL) \
-  DEF(char *, Delimit, ":") \
-  DEF(long,  DebugLevel, 0)
+  DEF(const char *, Delimit, ":") \
+  DEF(long,  DebugLevel, 0) \
+  DEF(int,  NoLineSupp, FALSE)
 
 #define STATE_VARS \
   DEF(enum ItState, ItState, itOff) /* Are we doing italics? */ \
@@ -290,29 +300,30 @@ DEBUGBITS(Debug_BIT)
   DEF(int, InHeader, TRUE)  /* Whether we're in the header */ \
   DEF(int, VerbMode, FALSE) /* Whether we're in complete ignore-mode */ \
   DEF(long, MathMode, 0)     /* Whether we're in math mode or not */ \
-  DEF(char *, VerbStr, "")   /* String we'll terminate verbmode upon */ \
+  DEF(const char *, VerbStr, "")   /* String we'll terminate verbmode upon */ \
   DEF(unsigned long, ErrPrint, 0)    /* # errors printed */ \
   DEF(unsigned long, WarnPrint, 0)   /* # warnings printed */ \
-  DEF(unsigned long, UserSupp, 0)    /* # user suppressed warnings */
+  DEF(unsigned long, UserSupp, 0)    /* # user suppressed warnings */ \
+  DEF(unsigned long, LineSupp, 0)    /* # warnings suppressed on a single line */
 
 #define DEF(type, name, value) extern type name;
-     OPTION_DEFAULTS STATE_VARS
+OPTION_DEFAULTS STATE_VARS
 #undef DEF
-     extern struct Stack CharStack, InputStack, EnvStack;
+extern struct Stack CharStack, InputStack, EnvStack;
 
-     enum Quote
-     {
-         quLogic,
-         quTrad
-     };
+enum Quote
+{
+    quLogic,
+    quTrad
+};
 
-     extern enum Quote Quote;
-     extern int StdInTTY, StdOutTTY, UsingStdIn;
+extern enum Quote Quote;
+extern int StdInTTY, StdOutTTY, UsingStdIn;
 
-     int main(int argc, char **argv);
-     void PrintPrgErr(enum PrgErrNum, ...);
-     void ErrPrintf(const char *fmt, ...);
+int main(int argc, char **argv);
+void PrintPrgErr(enum PrgErrNum, ...);
+void ErrPrintf(const char *fmt, ...);
 
-     extern char *PrgName;
+extern char *PrgName;
 
 #endif /* CHKTEX_H */
