@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Contact the author at:
  *              Jens Berger
@@ -33,6 +33,18 @@
 #include "Utility.h"
 #include "Resource.h"
 
+#if HAVE_PCRE
+
+#include <pcre.h>
+
+/* 3*(max_capturing_groups+1) */
+#define OVECCOUNT 3*(9+1)
+
+pcre** RegexArray = NULL;
+int NumRegexes = 0;
+
+#endif
+
 /***************************** ERROR MESSAGES ***************************/
 
 #undef MSG
@@ -42,14 +54,17 @@ struct ErrMsg LaTeXMsgs[emMaxFault + 1] = {
     ERRMSGS {emMaxFault, etErr, iuOK, 0, INTERNFAULT}
 };
 
-#define istex(c)        (isalpha(c) || (AtLetter && (c == '@')))
+#define istex(c)        (isalpha((unsigned char)c) || (AtLetter && (c == '@')))
 #define CTYPE(func) \
 static int my_##func(int c) \
 { \
    return(func(c)); \
 }
 
-#define INUSE(c)        (LaTeXMsgs[(enum ErrNum) c].InUse == iuOK)
+#define SUPPRESSED_ON_LINE(c)  (LineSuppressions & (1LL<<c))
+
+#define INUSE(c) \
+    ((LaTeXMsgs[(enum ErrNum) c].InUse == iuOK) && !SUPPRESSED_ON_LINE(c))
 
 #define PSERR2(pos,len,err,a,b) \
     PrintError(CurStkName(&InputStack), RealBuf, pos, len, Line, err, a, b)
@@ -98,6 +113,18 @@ static const char LTX_GenPunc[] = { ',', ';', 0 };
  */
 static const char LTX_SmallPunc[] = { '.', ',', 0 };
 
+/*
+ * String used to delimit a line suppression.  This string must be
+ * followed immediately by the number of the warning to be suppressed.
+ * If more than one warning is to be suppressed, then multiple copies
+ * of LineSuppDelim+number must be used.
+ */
+const char *LineSuppDelim = "chktex ";
+
+/*
+ * A bit field used to hold the suppressions for the current line.
+ */
+static unsigned long long LineSuppressions;
 
 static unsigned long Line;
 
@@ -106,6 +133,7 @@ static char *LineCpy;
 static char *BufPtr;
 
 static int ItFlag = efNone;
+static int MathFlag = efNone;
 
 NEWBUF(Buf, BUFSIZ);
 NEWBUF(CmdBuffer, BUFSIZ);
@@ -218,8 +246,10 @@ static char *GetLTXArg(char *SrcBuf, char *OrigDest, const int Until,
 
         if (Retval && (*OrigDest == '{') && (Until == GET_STRIP_TOKEN))
         {
-            strcpy(OrigDest, OrigDest + 1);
-            OrigDest[strlen(OrigDest) - 1] = 0;
+            int len = strlen(OrigDest+1);
+            memmove(OrigDest, OrigDest + 1, len  + 1);
+            /* Strip the last '}' off */
+            OrigDest[len-1] = 0;
         }
         break;
     default:
@@ -256,7 +286,11 @@ static char *MakeCpy(void)
 
 static char *PreProcess(void)
 {
-    /* First, kill comments. */
+    /* Reset any line suppressions  */
+
+    LineSuppressions = 0LL;
+
+    /* Kill comments. */
 
     char *TmpPtr;
 
@@ -266,10 +300,29 @@ static char *PreProcess(void)
 
     while ((TmpPtr = strchr(TmpPtr, '%')))
     {
-        if (TmpPtr[-1] != '\\')
+        char *EscapePtr = TmpPtr;
+        int NumBackSlashes = 0;
+        while (EscapePtr != Buf && EscapePtr[-1] == '\\')
+        {
+            ++NumBackSlashes;
+            --EscapePtr;
+        }
+
+        /* If there is an even number of backslashes, then it's a comment. */
+        if ((NumBackSlashes % 2) == 0)
         {
             PSERR(TmpPtr - Buf, 1, emComment);
             *TmpPtr = 0;
+            /* Check for line suppressions */
+            if (!NoLineSupp)
+            {
+                ++TmpPtr;       /* move past NUL terminator */
+                while ((TmpPtr = strcasestr(TmpPtr, LineSuppDelim))) {
+                    TmpPtr += STRLEN(LineSuppDelim);
+                    int error = atoi(TmpPtr);
+                    LineSuppressions |= (1LL<<error);
+                }
+            }
             break;
         }
         TmpPtr++;
@@ -363,9 +416,9 @@ static enum DotLevel CheckDots(char *PrePtr, char *PstPtr)
 
 }
 
-static char *Dot2Str(enum DotLevel dl)
+static const char *Dot2Str(enum DotLevel dl)
 {
-    char *Retval = INTERNFAULT;
+    const char *Retval = INTERNFAULT;
     switch (dl)
     {
     case dtUnknown:
@@ -517,7 +570,7 @@ static void PerformBigCmd(char *CmdPtr)
             PSERR(CmdPtr - Buf, CmdLen, emNoArgFound);
     }
 
-    if (HasWord(CmdBuffer, &NotPreSpaced) && isspace(CmdPtr[-1]))
+    if (HasWord(CmdBuffer, &NotPreSpaced) && isspace((unsigned char)CmdPtr[-1]))
         PSERRA(CmdPtr - Buf - 1, 1, emRemPSSpace, CmdBuffer);
 
     if ((TmpPtr = HasWord(CmdBuffer, &NoCharNext)))
@@ -590,8 +643,8 @@ static void PerformBigCmd(char *CmdPtr)
         dotlev = CheckDots(CmdPtr, BufPtr);
         if (dotlev && (dotlev != realdl))
         {
-            TmpPtr = Dot2Str(dotlev);
-            PSERRA(CmdPtr - Buf, CmdLen, emEllipsis, TmpPtr);
+            const char *cTmpPtr = Dot2Str(dotlev);
+            PSERRA(CmdPtr - Buf, CmdLen, emEllipsis, cTmpPtr);
         }
     }
 
@@ -622,7 +675,7 @@ static void CheckAbbrevs(const char *Buffer)
         for (i = Abbrev.MaxLen; i >= 0; i--)
         {
             *--TmpPtr = *AbbPtr--;
-            if (!isalpha(*AbbPtr) && HasWord(TmpPtr, &Abbrev))
+            if (!isalpha((unsigned char)*AbbPtr) && HasWord(TmpPtr, &Abbrev))
                 PSERR(Buffer - Buf + 1, 1, emInterWord);
             if (!*AbbPtr)
                 break;
@@ -644,6 +697,115 @@ static void CheckRest(void)
 
     /* Search for user-specified warnings */
 
+#if ! HAVE_PCRE
+
+    if (INUSE(emUserWarnRegex) && UserWarnRegex.Stack.Used > 0)
+    {
+        PrintPrgErr(pmNoRegExp);
+        ClearWord( &UserWarnRegex );
+    }
+
+#else
+
+    if (INUSE(emUserWarnRegex) && UserWarnRegex.Stack.Used > 0)
+    {
+        const char *error;
+        int ErrOffset;
+        int len = strlen(TmpBuffer);
+        static int ovector[OVECCOUNT];
+
+        /* Compile all regular expressions if not already compiled. */
+        if ( !RegexArray && UserWarnRegex.Stack.Used > 0 )
+        {
+            RegexArray = (pcre**)malloc( sizeof(pcre*) * UserWarnRegex.Stack.Used );
+            if (!RegexArray)
+            {
+                /* Allocation failed. */
+                PrintPrgErr(pmNoRegexMem);
+                ClearWord(&UserWarnRegex);
+                NumRegexes = 0;
+            }
+            else
+            {
+                NumRegexes = 0;
+                FORWL(Count, UserWarnRegex)
+                {
+                    char *pattern = UserWarnRegex.Stack.Data[Count];
+                    RegexArray[NumRegexes] = pcre_compile( pattern, 0, &error, &ErrOffset, NULL);
+
+                    /* Compilation failed: print the error message */
+                    if (RegexArray[NumRegexes] == NULL)
+                    {
+                        /* TODO: decide whether a non-compiling regex should completely stop, or just be ignored */
+                        PrintPrgErr(pmRegexCompileFailed, pattern, ErrOffset, error);
+                    }
+                    else
+                    {
+                        ++NumRegexes;
+                        /* Only use the comment if it's at the very beginning. */
+                        if ( strncmp(pattern,"(?#",3) == 0 )
+                        {
+                            char *CommentEnd = strchr(pattern, ')');
+                            *CommentEnd = '\0';
+                            /* We're leaking a little here, but this was never freed until exit anyway... */
+                            UserWarnRegex.Stack.Data[Count] = pattern+3;
+                        }
+                        else
+                        {
+                            ((char*)UserWarnRegex.Stack.Data[Count])[0] = '\0';
+                        }
+                    }
+                }
+            }
+        }
+
+        for (Count = 0; Count < NumRegexes; ++Count)
+        {
+            int offset = 0;
+            while (offset < len)
+            {
+
+                const int rc = pcre_exec( RegexArray[Count], NULL, TmpBuffer, len,
+                                          offset, 0, ovector, OVECCOUNT );
+
+                /* Matching failed: handle error cases */
+                if (rc < 0)
+                {
+                    switch(rc)
+                    {
+                        case PCRE_ERROR_NOMATCH:
+                            /* no match, no problem */
+                            break;
+                        default:
+                            PrintPrgErr(pmRegexMatchingError, rc);
+                            break;
+                    }
+                    offset = len; /* break out of loop */
+                }
+                else
+                {
+                    char *ErrMessage = UserWarnRegex.Stack.Data[Count];
+                    if ( strlen(ErrMessage) > 0 )
+                    {
+                        /* User specified error message */
+                        PSERR2(ovector[0], ovector[1]-ovector[0], emUserWarnRegex,
+                               strlen(ErrMessage), ErrMessage);
+                    }
+                    else
+                    {
+                        /* Default -- show the match */
+                        PSERR2(ovector[0], ovector[1]-ovector[0], emUserWarnRegex,
+                               ovector[1]-ovector[0], TmpBuffer+ovector[0]);
+                    }
+                    offset = ovector[1];
+                }
+            }
+        }
+    }
+
+#endif
+
+
     if (INUSE(emUserWarn))
     {
         strcpy(TmpBuffer, Buf);
@@ -654,11 +816,10 @@ static void CheckRest(void)
                  UsrPtr++)
             {
                 CmdLen = strlen(UserWarn.Stack.Data[Count]);
-                PSERR(UsrPtr - TmpBuffer, CmdLen, emUserWarn);
+                PSERRA(UsrPtr - TmpBuffer, CmdLen, emUserWarn, UserWarn.Stack.Data[Count]);
             }
         }
 
-
         strlwr(TmpBuffer);
 
         FORWL(Count, UserWarnCase)
@@ -668,7 +829,7 @@ static void CheckRest(void)
                  UsrPtr++)
             {
                 CmdLen = strlen(UserWarnCase.Stack.Data[Count]);
-                PSERR(UsrPtr - TmpBuffer, CmdLen, emUserWarn);
+                PSERRA(UsrPtr - TmpBuffer, CmdLen, emUserWarn, UserWarnCase.Stack.Data[Count]);
             }
         }
     }
@@ -702,9 +863,9 @@ static void CheckDash(void)
     {
         if (LATEX_SPACE(*PrePtr) && LATEX_SPACE(*TmpPtr))
             wl = &WordDash;
-        if (isdigit(*PrePtr) && isdigit(*TmpPtr))
+        if (isdigit((unsigned char)*PrePtr) && isdigit((unsigned char)*TmpPtr))
             wl = &NumDash;
-        if (isalpha(*PrePtr) && isalpha(*TmpPtr))
+        if (isalpha((unsigned char)*PrePtr) && isalpha((unsigned char)*TmpPtr))
             wl = &HyphDash;
 
         if (wl)
@@ -747,6 +908,7 @@ static void HandleBracket(int Char)
             if ((ei = PopErr(&CharStack)))
             {
                 Match = MatchBracket(*(ei->Data));
+                /* Return italics to proper state */
                 if (ei->Flags & efNoItal)
                 {
                     if (ItState == itOn)
@@ -762,6 +924,17 @@ static void HandleBracket(int Char)
                 }
                 else if (ei->Flags & efItal)
                     ItState = TRUE;
+
+                /* Same for math mode */
+                if (ei->Flags & efMath)
+                {
+                    MathMode = 1;
+                }
+                else if (ei->Flags & efNoMath)
+                {
+                    MathMode = 0;
+                }
+
                 FreeErrInfo(ei);
             }
             else
@@ -791,12 +964,22 @@ static void HandleBracket(int Char)
                     switch (ItFlag)
                     {
                     default:
-                        ei->Flags = ItFlag;
+                        ei->Flags |= ItFlag;
                         ItFlag = efNone;
                         break;
                     case efNone:
                         ei->Flags |= ItState ? efItal : efNoItal;
                     }
+
+                    switch (MathFlag)
+                    {
+                    default:
+                        ei->Flags |= MathFlag;
+                        MathFlag = efNone;
+                        break;
+                    case efNone:
+                        ei->Flags |= MathMode ? efMath : efNoMath;
+                    }
                 }
             }
 
@@ -841,20 +1024,25 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
 
         BufPtr = SkipVerb();
 
+        if (!VerbMode)
+        {
+            CheckRest();
+        }
+
         while (BufPtr && *BufPtr)
         {
             PrePtr = BufPtr - 1;
             Char = *BufPtr++;
-            if (isspace(Char))
+            if (isspace((unsigned char)Char))
                 Char = ' ';
 
             switch (Char)
             {
             case '~':
                 TmpPtr = NULL;
-                if (isspace(*PrePtr))
+                if (isspace((unsigned char)*PrePtr))
                     TmpPtr = PrePtr;
-                else if (isspace(*BufPtr))
+                else if (isspace((unsigned char)*BufPtr))
                     TmpPtr = BufPtr;
 
                 if (TmpPtr)
@@ -868,14 +1056,14 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                 SKIP_BACK(TmpPtr, TmpC,
                           (LATEX_SPACE(TmpC) || strchr("{}$", TmpC)));
 
-                if (isdigit(*TmpPtr))
+                if (isdigit((unsigned char)*TmpPtr))
                 {
                     TmpPtr = BufPtr;
 
                     SKIP_AHEAD(TmpPtr, TmpC,
                                (LATEX_SPACE(TmpC) || strchr("{}$", TmpC)));
 
-                    if (isdigit(*TmpPtr))
+                    if (isdigit((unsigned char)*TmpPtr))
                         HERE(1, emUseTimes);
                 }
                 /* FALLTHRU */
@@ -931,7 +1119,7 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
             case 'W':          /* case 'X': */
             case 'Y':
             case 'Z':
-                if (!isalpha(*PrePtr) && (*PrePtr != '\\') && MathMode)
+                if (!isalpha((unsigned char)*PrePtr) && (*PrePtr != '\\') && MathMode)
                 {
                     TmpPtr = BufPtr;
                     CmdPtr = CmdBuffer;
@@ -940,7 +1128,7 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                         *CmdPtr++ = Char;
                         Char = *TmpPtr++;
                     }
-                    while (isalpha(Char));
+                    while (isalpha((unsigned char)Char));
 
                     *CmdPtr = 0;
 
@@ -966,9 +1154,10 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
             case '.':
                 if ((Char == *BufPtr) && (Char == BufPtr[1]))
                 {
+                    const char *cTmpPtr;
                     dotlev = CheckDots(&PrePtr[1], &BufPtr[2]);
-                    TmpPtr = Dot2Str(dotlev);
-                    HEREA(3, emEllipsis, TmpPtr);
+                    cTmpPtr = Dot2Str(dotlev);
+                    HEREA(3, emEllipsis, cTmpPtr);
                 }
 
                 /* Regexp: "([^A-Z@.])\.[.!?:;]*\s+[a-z]" */
@@ -977,11 +1166,11 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                 SKIP_AHEAD(TmpPtr, TmpC, strchr(LTX_GenPunc, TmpC));
                 if (LATEX_SPACE(*TmpPtr))
                 {
-                    if (!isupper(*PrePtr) && (*PrePtr != '@') &&
+                    if (!isupper((unsigned char)*PrePtr) && (*PrePtr != '@') &&
                         (*PrePtr != '.'))
                     {
                         SKIP_AHEAD(TmpPtr, TmpC, LATEX_SPACE(TmpC));
-                        if (islower(*TmpPtr))
+                        if (islower((unsigned char)*TmpPtr))
                             PSERR(BufPtr - Buf, 1, emInterWord);
                         else
                             CheckAbbrevs(&BufPtr[-1]);
@@ -995,14 +1184,14 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
             case ';':
                 /* Regexp: "[A-Z][A-Z][.!?:;]\s+" */
 
-                if (isspace(*BufPtr) && isupper(*PrePtr) &&
-                    (isupper(PrePtr[-1]) || (Char != '.')))
+                if (isspace((unsigned char)*BufPtr) && isupper((unsigned char)*PrePtr) &&
+                    (isupper((unsigned char)PrePtr[-1]) || (Char != '.')))
                     HERE(1, emInterSent);
 
                 /* FALLTHRU */
             case ',':
-                if (isspace(*PrePtr) &&
-                    !(isdigit(*BufPtr) &&
+                if (isspace((unsigned char)*PrePtr) &&
+                    !(isdigit((unsigned char)*BufPtr) &&
                       ((BufPtr[-1] == '.') || (BufPtr[-1] == ','))))
                     PSERR(PrePtr - Buf, 1, emSpacePunct);
 
@@ -1049,8 +1238,8 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                 switch (Char)
                 {
                 case '\'':
-                    if (isalpha(*TmpPtr) &&
-                        (strchr(LTX_GenPunc, *PrePtr) || isspace(*PrePtr)))
+                    if (isalpha((unsigned char)*TmpPtr) &&
+                        (strchr(LTX_GenPunc, *PrePtr) || isspace((unsigned char)*PrePtr)))
                         HERE(TmpPtr - BufPtr + 1, emBeginQ);
 
                     /* Now check quote style */
@@ -1060,7 +1249,7 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
 
                     {
                         char *WordPtr = PrePtr;
-                        SKIP_BACK(WordPtr, TmpC, (isalnum(TmpC) ||
+                        SKIP_BACK(WordPtr, TmpC, (isalnum((unsigned char)TmpC) ||
                                                   strchr(LTX_GenPunc, TmpC)));
 
                         if (*WordPtr != '`')
@@ -1079,8 +1268,8 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
 
                     break;
                 case '`':
-                    if (isalpha(*PrePtr) &&
-                        (strchr(LTX_GenPunc, *TmpPtr) || isspace(*TmpPtr)))
+                    if (isalpha((unsigned char)*PrePtr) &&
+                        (strchr(LTX_GenPunc, *TmpPtr) || isspace((unsigned char)*TmpPtr)))
                         HERE(TmpPtr - BufPtr + 1, emEndQ);
                     break;
                 }
@@ -1128,9 +1317,9 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
 
                     ErrPtr = TmpPtr;
 
-                    if (isalpha(*TmpPtr))
+                    if (isalpha((unsigned char)*TmpPtr))
                         pstcb = &my_isalpha;
-                    else if (isdigit(*TmpPtr))
+                    else if (isdigit((unsigned char)*TmpPtr))
                         pstcb = &my_isdigit;
                     else
                         break;
@@ -1163,7 +1352,7 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                 {
                     PSERR(BufPtr - Buf, 1, emSpaceTerm);
                 }
-                else if ((*BufPtr == '\\') && (!isalpha(BufPtr[1])) &&
+                else if ((*BufPtr == '\\') && (!isalpha((unsigned char)BufPtr[1])) &&
                          (!LATEX_SPACE(BufPtr[1])))
                     PSERR(BufPtr - Buf, 2, emNotIntended);
 
@@ -1173,8 +1362,8 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                 break;
 
             case '(':
-                if (!(!*PrePtr || LATEX_SPACE(*PrePtr) || isdigit(*PrePtr)) ||
-                    strchr("([{`~", *PrePtr))
+                if (*PrePtr && !LATEX_SPACE(*PrePtr) && !isdigit((unsigned char)*PrePtr)
+                    && !strchr("([{`~", *PrePtr))
                 {
                     if (PrePtr[-1] != '\\')     /* Short cmds */
                     {
@@ -1185,16 +1374,16 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
                                    "in front of");
                     }
                 }
-                if (isspace(*BufPtr))
+                if (isspace((unsigned char)*BufPtr))
                     PSERRA(BufPtr - Buf, 1, emNoSpaceParen, "after");
                 HandleBracket(Char);
                 break;
 
             case ')':
-                if (isspace(*PrePtr))
+                if (isspace((unsigned char)*PrePtr))
                     PSERRA(BufPtr - Buf - 1, 1, emNoSpaceParen,
                            "in front of");
-                if (isalpha(*BufPtr))
+                if (isalpha((unsigned char)*BufPtr))
                     PSERRA(BufPtr - Buf, 1, emSpaceParen, "after");
                 HandleBracket(Char);
                 break;
@@ -1217,10 +1406,6 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
             }
         }
 
-        if (!VerbMode)
-        {
-            CheckRest();
-        }
     }
 
     return (TRUE);
@@ -1231,7 +1416,7 @@ int FindErr(const char *_RealBuf, const unsigned long _Line)
  * suffix should be put, e.g. "warning%s". Watch your %'s!
  */
 
-static void Transit(FILE * fh, unsigned long Cnt, char *Str)
+static void Transit(FILE * fh, unsigned long Cnt, const char *Str)
 {
     switch (Cnt)
     {
@@ -1294,7 +1479,8 @@ void PrintStatus(unsigned long Lines)
     {
         Transit(stderr, ErrPrint, "error%s printed; ");
         Transit(stderr, WarnPrint, "warning%s printed; ");
-        Transit(stderr, UserSupp, "user suppressed warning%s printed.\n");
+        Transit(stderr, UserSupp, "user suppressed warning%s; ");
+        Transit(stderr, LineSupp, "line suppressed warning%s.\n");
     }
 }
 
@@ -1316,7 +1502,8 @@ void PrintStatus(unsigned long Lines)
  *  %n  - warning Number
  *  %u  - an Underlining line (like the one which appears when using -v1)
  *  %r  - part of line in front of error ('S' - 1)
- *  %s  - part of line which contains error (String) *  %t  - part of line after error ('S' + 1)
+ *  %s  - part of line which contains error (String)
+ *  %t  - part of line after error ('S' + 1)
  */
 
 
@@ -1340,7 +1527,11 @@ PrintError(const char *File, const char *String,
         switch (LaTeXMsgs[Error].InUse)
         {
         case iuOK:
-            do
+            if (SUPPRESSED_ON_LINE(Error))
+            {
+                LineSupp++;
+            }
+            else
             {
                 Context = LaTeXMsgs[Error].Context;
 
@@ -1449,7 +1640,6 @@ PrintError(const char *File, const char *String,
                 }
                 fputs(LastNorm, OutputFile);
             }
-            while (FALSE);
             break;
         case iuNotUser:
             UserSupp++;
@@ -1468,7 +1658,7 @@ PrintError(const char *File, const char *String,
 
 static enum ErrNum PerformCommand(const char *Cmd, char *Arg)
 {
-    char *Argument = "";
+    const char *Argument = "";
     enum ErrNum en = emMinFault;
     int TmpC;
 
@@ -1476,8 +1666,7 @@ static enum ErrNum PerformCommand(const char *Cmd, char *Arg)
         AtLetter = TRUE;
     else if (!strcmp(Cmd, "\\makeatother"))
         AtLetter = FALSE;
-    else if (InputFiles &&
-             !(strcmp(Cmd, "\\input") && strcmp(Cmd, "\\include")))
+    else if (InputFiles && !(strcmp(Cmd, "\\input") && strcmp(Cmd, "\\include")))
     {
         SKIP_AHEAD(Arg, TmpC, LATEX_SPACE(TmpC));
         if (*Arg == '{')        /* } */
@@ -1493,6 +1682,24 @@ static enum ErrNum PerformCommand(const char *Cmd, char *Arg)
     }
     else if (HasWord(Cmd, &Primitives))
         en = emTeXPrim;
+    else if (HasWord(Cmd, &MathCmd))
+    {
+        SKIP_AHEAD(Arg, TmpC, LATEX_SPACE(TmpC));
+        if (*Arg == '{')
+        {
+            MathFlag = MathMode ? efMath : efNoMath;
+            MathMode = 1;
+        }
+    }
+    else if (HasWord(Cmd, &TextCmd))
+    {
+        SKIP_AHEAD(Arg, TmpC, LATEX_SPACE(TmpC));
+        if (*Arg == '{')
+        {
+            MathFlag = MathMode ? efMath : efNoMath;
+            MathMode = 0;
+        }
+    }
     else if (*Cmd == '\\')
     {
         /* Quicker check of single lettered commands. */
