@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Contact the author at:
  *              Jens Berger
@@ -76,7 +76,7 @@
  MSG(emUseOtherQuote, etWarn, iuOK, ctNone,\
      "Use \"'\" (ASCII 39) instead  of \"\" (ASCII 180).")\
  MSG(emUserWarn, etWarn, iuOK, ctNone,\
-     "User-specified pattern found.")\
+     "User-specified pattern found: %s.")\
  MSG(emNotIntended, etWarn, iuNotSys, ctNone,\
      "This command might not be intended.")\
  MSG(emComment, etMsg, iuNotSys, ctNone,\
@@ -122,7 +122,9 @@
  MSG(emRemPSSpace, etWarn, iuOK, ctOutMath,\
      "You should remove spaces in front of `%s'") \
  MSG(emNoCharMean, etWarn, iuOK, ctNone,\
-     "`%s' is normally not followed by `%c'.")
+     "`%s' is normally not followed by `%c'.")\
+ MSG(emUserWarnRegex, etWarn, iuOK, ctNone,\
+     "User Regex: %.*s.")
 
 #undef MSG
 #define MSG(num, type, inuse, ctxt, text) num,
@@ -162,7 +164,7 @@ struct ErrMsg
     /* Requirements posed upon environment */
     enum Context Context;
 
-    char *Message;
+    const char *Message;
 };
 
 enum ItState
