@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Contact the author at:
  *              Jens Berger
@@ -38,17 +38,18 @@
 #define RESOURCE_INFO \
   LIST(Silent)     LIST(Linker)     LIST(IJAccent) LIST(Italic)\
   LIST(ItalCmd)    LIST(PostLink)   LIST(WipeArg)  LIST(VerbEnvir)\
-  LIST(MathEnvir)  LIST(MathRoman)  LIST(HyphDash) LIST(NumDash)\
-  LIST(WordDash)   LIST(CenterDots) LIST(LowDots)  LIST(OutFormat)\
+  LIST(MathEnvir)  LIST(MathCmd)    LIST(TextCmd)  LIST(MathRoman)\
+  LIST(HyphDash)   LIST(NumDash)    LIST(WordDash)\
+  LIST(CenterDots) LIST(LowDots)    LIST(OutFormat)\
   LIST(Primitives) LIST(NotPreSpaced) LIST(NonItalic) LIST(NoCharNext)\
   LNEMPTY(CmdLine) LNEMPTY(TeXInputs)\
-  LCASE(Abbrev)    LCASE(UserWarn)\
+  LCASE(Abbrev)    LCASE(UserWarn)  LIST(UserWarnRegex)\
   KEY(VerbClear, "|") KEY(QuoteStyle, "Traditional")  KEY(TabSize, "8")
 
 #define LIST(a) extern struct WordList a;
 #define LNEMPTY LIST
 #define LCASE(a) LIST(a) LIST(a ## Case)
-#define KEY(a,def) extern char *a;
+#define KEY(a,def) extern const char *a;
 
 RESOURCE_INFO
 #undef KEY
