@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  Contact the author at:
  *              Jens Berger
@@ -95,6 +95,9 @@ struct FileNode
 /* Rotates x n bits right (should be an int, long, etc.) */
 #define ROTATER(x,n) ((x>>n) | (x<<((CHAR_BIT*sizeof(x)) - n)))
 
+/* Subtract 1 because sizeof includes the null terminator. */
+#define STRLEN(x)  (sizeof(x)/sizeof(x[0]) - 1)
+
 int fexists(const char *Filename);
 
 void *sfmemset(void *to, int c, long n);
@@ -105,7 +108,7 @@ void strrep(char *String, const char From, const char To);
 void strxrep(char *Buf, const char *Prot, const char To);
 char *strip(char *String, const enum Strip What);
 void strwrite(char *To, const char *From, unsigned long Len);
-int strinfront(char *Str, char *Cmp);
+int strinfront(const char *Str, const char *Cmp);
 char *strdupx(const char *String, int Extra);
 void strmove(char *a, const char *b);
 
@@ -124,11 +127,11 @@ void *StkPop(struct Stack *Stack);
 void *StkTop(struct Stack *Stack);
 
 FILE *CurStkFile(struct Stack *stack);
-char *CurStkName(struct Stack *stack);
+const char *CurStkName(struct Stack *stack);
 unsigned long CurStkLine(struct Stack *stack);
 char *FGetsStk(char *Dest, unsigned long len, struct Stack *stack);
 int PushFileName(const char *Name, struct Stack *stack);
-int PushFile(char *, FILE *, struct Stack *);
+int PushFile(const char *, FILE *, struct Stack *);
 
 
 void FreeErrInfo(struct ErrInfo *ei);
